@@ -2,13 +2,13 @@
 #'
 #' @inheritParams Future-class
 #' 
-#' @param \dots Additional named elements passed to \code{\link{Future}()}.
+#' @param \dots Additional named elements passed to [Future()].
 #'
-#' @return An object of class \code{MultiprocessFuture}.
+#' @return An object of class `MultiprocessFuture`.
 #'
 #' @seealso
 #' To evaluate an expression using "multiprocess future", see function
-#' \code{\link{multiprocess}()}.
+#' [multiprocess()].
 #'
 #' @export
 #' @name MultiprocessFuture-class
@@ -16,6 +16,7 @@
 MultiprocessFuture <- function(expr = NULL, envir = parent.frame(), substitute = FALSE, ...) {
   if (substitute) expr <- substitute(expr)
 
-  f <- Future(expr = expr, envir = envir, substitute = FALSE, ...)
-  structure(f, class = c("MultiprocessFuture", class(f)))
+  future <- Future(expr = expr, envir = envir, substitute = FALSE, ...)
+  future <- structure(future, class = c("MultiprocessFuture", class(future)))
+  future
 }
