@@ -2,13 +2,13 @@
 #'
 #' @inheritParams Future-class
 #' 
-#' @param \dots Additional named elements passed to \code{\link{Future}()}.
+#' @param \dots Additional named elements passed to [Future()].
 #'
-#' @return An object of class \code{UniprocessFuture}.
+#' @return An object of class `UniprocessFuture`.
 #'
 #' @seealso
 #' To evaluate an expression using "uniprocess future", see functions
-#' \code{\link{uniprocess}()}.
+#' [uniprocess()].
 #'
 #' @export
 #' @name UniprocessFuture-class
@@ -31,9 +31,9 @@ UniprocessFuture <- function(expr = NULL, envir = parent.frame(), substitute = F
   
   gp <- NULL
  
-  f <- Future(expr = expr, envir = envir, substitute = FALSE, lazy = lazy, asynchronous = FALSE, local = local, globals = globals, packages = packages, version = "1.8", ...)
-  f$.callResult <- TRUE
-  structure(f, class = c("UniprocessFuture", class(f)))
+  future <- Future(expr = expr, envir = envir, substitute = FALSE, lazy = lazy, asynchronous = FALSE, local = local, globals = globals, packages = packages, ...)
+  future <- structure(future, class = c("UniprocessFuture", class(future)))
+  future
 }
 
 
