@@ -3,7 +3,7 @@
 #' Creates a future that evaluates an \R expression or
 #' a future that calls an \R function with a set of arguments.
 #' How, when, and where these futures are evaluated can be configured
-#' using \code{\link{plan}()} such that it is evaluated in parallel on,
+#' using [plan()] such that it is evaluated in parallel on,
 #' for instance, the current machine, on a remote machine, or via a
 #' job queue on a compute cluster.
 #' Importantly, any \R code using futures remains the same regardless
@@ -17,52 +17,52 @@
 #' @param \dots Reserved for internal use only.
 #'
 #' @return
-#' \code{f <- future(expr)} creates a \link{Future} \code{f} that evaluates expression \code{expr}, the value of the future is retrieved using \code{v <- value(f)}.
+#' `f <- future(expr)` creates a [Future] `f` that evaluates expression `expr`, the value of the future is retrieved using `v <- value(f)`.
 #'
 #' @details
 #' The state of a future is either unresolved or resolved.
 #' The value of a future can be retrieved using \code{v <- \link{value}(f)}.
-#' Querying the value of a non-resolved future will \emph{block} the call
+#' Querying the value of a non-resolved future will _block_ the call
 #' until the future is resolved.
 #' It is possible to check whether a future is resolved or not
 #' without blocking by using \code{\link{resolved}(f)}.
 #'
 #' For a future created via a future assignment
-#' (\code{x \%<-\% value} or \code{futureAssign("x", value)}), the value
+#' (`x %<-% value` or `futureAssign("x", value)`), the value
 #' is bound to a promise, which when queried will internally call
-#' \code{\link{value}()}  on the future and which will then be resolved
+#' [value()]  on the future and which will then be resolved
 #' into a regular variable bound to that value.  For example, with future
-#' assignment \code{x \%<-\% value}, the first time variable \code{x} is
+#' assignment `x %<-% value`, the first time variable `x` is
 #' queried the call blocks if (and only if) the future is not yet resolved.
-#' As soon as it is resolved, and any succeeding queries, querying \code{x}
+#' As soon as it is resolved, and any succeeding queries, querying `x`
 #' will immediately give the value.
 #'
-#' The future assignment construct \code{x \%<-\% value} is not a formal
-#' assignment per se, but a binary infix operator on objects \code{x}
-#' and expression \code{value}.  However, by using non-standard evaluation,
+#' The future assignment construct `x %<-% value` is not a formal
+#' assignment per se, but a binary infix operator on objects `x`
+#' and expression `value`.  However, by using non-standard evaluation,
 #' this constructs can emulate an assignment operator similar to
-#' \code{x <- value}. Due to \R's precedence rules of operators,
+#' `x <- value`. Due to \R's precedence rules of operators,
 #' future expressions often need to be explicitly bracketed, e.g.
-#' \code{x \%<-\% { a + b }}.
+#' `x %<-% { a + b }`.
 #'
 #'
 #' @section Eager or lazy evaluation:
-#' By default, a future is resolved using \emph{eager} evaluation
-#' (\code{lazy = FALSE}).  This means that the expression starts to
+#' By default, a future is resolved using _eager_ evaluation
+#' (`lazy = FALSE`).  This means that the expression starts to
 #' be evaluated as soon as the future is created.
 #'
-#' As an alternative, the future can be resolved using \emph{lazy}
-#' evaluation (\code{lazy = TRUE}).  This means that the expression
+#' As an alternative, the future can be resolved using _lazy_
+#' evaluation (`lazy = TRUE`).  This means that the expression
 #' will only be evaluated when the value of the future is requested.
-#' \emph{Note that this means that the expression may not be evaluated
-#' at all - it is guaranteed to be evaluated if the value is requested}.
+#' _Note that this means that the expression may not be evaluated
+#' at all - it is guaranteed to be evaluated if the value is requested_.
 #'
 #' For future assignments, lazy evaluation can be controlled via the
-#' \code{\%lazy\%} operator, e.g. \code{x \%<-\% { expr } \%lazy\% TRUE}.
+#' `%lazy%` operator, e.g. `x %<-% { expr } %lazy% TRUE`.
 #'
 #'
 #' @section Globals used by future expressions:
-#' Global objects (short \emph{globals}) are objects (e.g. variables and
+#' Global objects (short _globals_) are objects (e.g. variables and
 #' functions) that are needed in order for the future expression to be
 #' evaluated while not being local objects that are defined by the future
 #' expression. For example, in
@@ -70,20 +70,20 @@
 #'   a <- 42
 #'   f <- future({ b <- 2; a * b })
 #' }
-#' variable \code{a} is a global of future assignment \code{f} whereas
-#' \code{b} is a local variable.
+#' variable `a` is a global of future assignment `f` whereas
+#' `b` is a local variable.
 #' In order for the future to be resolved successfully (and correctly),
 #' all globals need to be gathered when the future is created such that
 #' they are available whenever and wherever the future is resolved.
 #'
-#' The default behavior (\code{globals = TRUE}),
+#' The default behavior (`globals = TRUE`),
 #' is that globals are automatically identified and gathered.
 #' More precisely, globals are identified via code inspection of the
-#' future expression \code{expr} and their values are retrieved with
-#' environment \code{envir} as the starting point (basically via
-#' \code{get(global, envir = envir, inherits = TRUE)}).
-#' \emph{In most cases, such automatic collection of globals is sufficient
-#' and less tedious and error prone than if they are manually specified}.
+#' future expression `expr` and their values are retrieved with
+#' environment `envir` as the starting point (basically via
+#' `get(global, envir = envir, inherits = TRUE)`).
+#' _In most cases, such automatic collection of globals is sufficient
+#' and less tedious and error prone than if they are manually specified_.
 #'
 #' However, for full control, it is also possible to explicitly specify
 #' exactly which the globals are by providing their names as a character
@@ -125,8 +125,8 @@
 #'   x <- rnorm(1000)
 #'   f <- future({ median(x) })
 #' }
-#' variable \code{x} and \code{median()} are globals, but only \code{x}
-#' is exported whereas \code{median()}, which is part of the \pkg{stats}
+#' variable `x` and `median()` are globals, but only `x`
+#' is exported whereas `median()`, which is part of the \pkg{stats}
 #' package, is not exported.  Instead it is made sure that the \pkg{stats}
 #' package is on the search path when the future expression is evaluated.
 #' Effectively, the above becomes
@@ -156,12 +156,12 @@
 #'
 #'
 #' Although rarely needed, a combination of automatic identification and manual
-#' specification of globals is supported via attributes \code{add} (to add
-#' false negatives) and \code{ignore} (to ignore false positives) on value
-#' \code{TRUE}.  For example, with
-#' \code{globals = structure(TRUE, ignore = "b", add = "a")} any globals
-#' automatically identified except \code{b} will be used in addition to
-#' global \code{a}.
+#' specification of globals is supported via attributes `add` (to add
+#' false negatives) and `ignore` (to ignore false positives) on value
+#' `TRUE`.  For example, with
+#' `globals = structure(TRUE, ignore = "b", add = "a")` any globals
+#' automatically identified except `b` will be used in addition to
+#' global `a`.
 #'
 #' When using future assignments, globals can be specified analogously
 #' using the \code{\link{\%globals\%}} operator, e.g.
@@ -175,41 +175,34 @@
 #'
 #' @seealso
 #' How, when and where futures are resolved is given by the
-#' \emph{future strategy}, which can be set by the end user using the
-#' \code{\link{plan}()} function.  The future strategy must not be
+#' _future strategy_, which can be set by the end user using the
+#' [plan()] function.  The future strategy must not be
 #' set by the developer, e.g. it must not be called within a package.
 #'
 #' @aliases futureCall
 #' @export
 #' @name future
-future <- function(expr, envir = parent.frame(), substitute = TRUE, globals = TRUE, packages = NULL, lazy = FALSE, seed = NULL, ...) {
+future <- function(expr, envir = parent.frame(), substitute = TRUE, globals = TRUE, packages = NULL, seed = FALSE, lazy = FALSE, ...) {
   if (substitute) expr <- substitute(expr)
 
-  ## Hidden argument 'evaluator':
-  ## The 'evaluator' is the function that creates a Future object.
-  ## The default evaluator function is given by plan().
-  makeFuture <- list(...)$evaluator
-  if (!is.null(makeFuture)) {
-    action <- get(Sys.getenv("R_CHECK_FUTURE_EVALUATOR", ".Defunct"), mode="function")
-    action(msg = "Argument 'evaluator' of future() was an internal argument and is now deprecated. Use plan() to set the \"evaluator\".")
-  }
-  if (is.null(makeFuture)) makeFuture <- plan("next")
-  if (!is.function(makeFuture)) {
-    stop("Argument 'evaluator' must be a function: ", typeof(makeFuture))
+  ## Argument 'evaluator' is defunct
+  if (!is.null(list(...)$evaluator)) {
+    .Defunct(msg = "Argument 'evaluator' of future() was an internal argument and is now defunct. Use plan() to set the \"evaluator\".")
   }
+  makeFuture <- plan("next")
+  ## Sanity check
+  stopifnot(is.function(makeFuture))
 
-  ## Call 'makeFuture' without passing 'evaluator' argument part of '...'
-  .makeFuture <- function(..., evaluator = NULL) makeFuture(...)
-  future <- .makeFuture(expr, substitute = FALSE,
-                        envir = envir,
-		        globals = globals, packages = packages,
-                        seed = seed,
-		        lazy = lazy,
-		        ...)
+  future <- makeFuture(expr, substitute = FALSE,
+                       envir = envir,
+                       globals = globals, packages = packages,
+                       seed = seed,
+                       lazy = lazy,
+                       ...)
 
   ## Assert that a future was returned
   if (!inherits(future, "Future")) {
-    stop(FutureError("Argument 'evaluator' specifies a function that does not return a Future object: ", paste(sQuote(class(future)), collapse = ", ")))
+    stop(FutureError("plan(\"next\") returned a function that does not return a Future object: ", paste(sQuote(class(future)), collapse = ", ")))
   }
 
   future
