@@ -3,23 +3,24 @@
 #' Gets all values in an environment, a list, or a list environment
 #' and returns an object of the same class (and dimensions).
 #' All future elements are replaced by their corresponding
-#' \code{value()} values.  For all other elements, the existing
+#' `value()` values.  For all other elements, the existing
 #' object is kept.
 #'
-#' @param x An environment, a list, or a list environment.
+#' @param x A [Future], an environment, a list, or a list environment.
 #'
 #' @param stdout If TRUE, captured standard output is relayed, otherwise note.
 #' 
 #' @param signal If TRUE, captured (\link[base]{conditions}) are relayed,
 #' otherwise not.
 #' 
-#' @param \dots Additional arguments passed to \code{value()} of each future.
+#' @param \dots Additional arguments passed to `value()` of each future.
 #'
-#' @return An object of same type as \code{x} and with the same names
+#' @return An object of same type as `x` and with the same names
 #' and/or dimensions, if set.
-#' If \code{signal} is TRUE and one of the futures produces an error, then
+#' If `signal` is TRUE and one of the futures produces an error, then
 #' that error is produced.
 #'
+#' @aliases value.list value.environment value.listenv
 #' @export
 values <- function(x, stdout = TRUE, signal = TRUE, ...) UseMethod("values")
 
@@ -80,3 +81,13 @@ values.listenv <- function(x, stdout = TRUE, signal = TRUE, ...) {
 values.Future <- function(x, ...) {
   value(x, ...)
 }
+
+
+#' @export
+value.list <- values.list
+
+#' @export
+value.environment <- values.environment
+
+#' @export
+value.listenv <- values.listenv
